package ncsu.course.android.broadcastchat4;

import java.util.ArrayList;
import java.util.Random;
import java.util.StringTokenizer;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.view.inputmethod.EditorInfo;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
 
/**  
 * This is the main Activity that displays the current chat session.
 */
public class BroadcastChat extends Activity {
    // Debugging
    private static final String TAG = "BcastChat";
    private static final boolean D = true;

    // Message types sent from the BluetoothChatService Handler
    public static final int MESSAGE_READ 	= 1;
    public static final int MESSAGE_WRITE 	= 2;
    public static final int MESSAGE_TOAST 	= 3;

    public static final String ROOM_LIST 			= "room_list";
    private static final long FIND_ROOM_TIME_OUT 	= 2000;
    private ArrayList<String> existingRoomList 		= null;
    private static final int RANDOM_WAIT_WIN 		= 500;
    
    // Key names received from the BroadcastChatService Handler
    public static final String TOAST = "toast";

    // Intent request codes
    private static final int INTENT_GET_USERNAME 	= 1;
    private static final int INTENT_CREATE_ROOM 	= 2;
    private static final int INTENT_FIND_ROOM		= 3;
    
    // Layout Views
    private TextView mTitle;
    private ListView 	mConversationView;
    private EditText 	mOutEditText;
    private Button 		mSendButton;

    // Array adapter for the conversation thread
    private ArrayAdapter<String> mConversationArrayAdapter;
    // String buffer for outgoing messages
    private StringBuffer mOutStringBuffer;
    // Member object for the chat services
    private BroadcastChatService mChatService = null;
    
    // message format
    private static final String CMD 	= "cmd=";
    private static final String ROOM 	= "room=";
    private static final String BODY 	= "body=";
    private static final String CMD_FIND_ROOM 		= "1";
    private static final String CMD_ROOM_NAME_ACK 	= "2";
    private static final String CMD_MESSAGE 		= "3";
    private static final String DELIMITER			= "||";
        
    //user and room name
    private String mUserName = null;
    private String mRoomName = null;
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if(D) Log.e(TAG, "+++ ON CREATE +++");

        // Set up the window layout
        requestWindowFeature(Window.FEATURE_CUSTOM_TITLE);
        setContentView(R.layout.main);
        
        getWindow().setFeatureInt(Window.FEATURE_CUSTOM_TITLE, R.layout.custom_title);
        // Set up the custom title
        mTitle = (TextView) findViewById(R.id.title_left_text); 
        mTitle.setText(R.string.app_name);
        mTitle = (TextView) findViewById(R.id.title_right_text);
    }

   
    public void onStart() {
        super.onStart();
        if(D) Log.e(TAG, "++ ON START ++");

        setupChat();
    }

    @Override
    public synchronized void onResume() {
        super.onResume();
        if(D) Log.e(TAG, "+ ON RESUME +");
        
        mChatService.start();
    }


    private void setupChat() {
        Log.d(TAG, "setupChat()");

        // Initialize the array adapter for the conversation thread
        mConversationArrayAdapter = new ArrayAdapter<String>(this, R.layout.message);
        mConversationView = (ListView) findViewById(R.id.in);
        mConversationView.setAdapter(mConversationArrayAdapter);

        // Initialize the compose field with a listener for the return key
        mOutEditText = (EditText) findViewById(R.id.edit_text_out);
        mOutEditText.setOnEditorActionListener(mWriteListener);

        // Initialize the send button with a listener that for click events
        mSendButton = (Button) findViewById(R.id.button_send);
        mSendButton.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
            	if(D) Log.e(TAG, "[sendButton clicked]");
                // Send a message using content of the edit text widget
                TextView view = (TextView) findViewById(R.id.edit_text_out);
                String message = view.getText().toString();
                sendMessage(message);                
            }
        });

        // Initialize the BluetoothChatService to perform bluetooth connections
        if(mChatService == null)
        	mChatService = new BroadcastChatService(this, mHandler);

        // Initialize the buffer for outgoing messages
        if(mOutStringBuffer == null)
        	mOutStringBuffer = new StringBuffer("");
    }
    
    
    public synchronized void onPause() {
        super.onPause();
        if(D) Log.e(TAG, "- ON PAUSE -");
        if (mChatService != null) mChatService.stop();
    }


    public void onStop() {
        super.onStop();
        if(D) Log.e(TAG, "-- ON STOP --");
        if (mChatService != null) mChatService.stop();
    }


    public void onDestroy() {
        super.onDestroy();
        if(D) Log.e(TAG, "--- ON DESTROY ---");
        // Stop the Broadcast chat services
        if (mChatService != null) mChatService.stop();
        
    }
    
    
    private void sendRoomNameAck(){
    	
    	if(mRoomName == null)
    		return;
    	
    	try {
    		
    		Random randomGenerator = new Random();
 		    int randomInt = randomGenerator.nextInt(RANDOM_WAIT_WIN);
 		    Thread.sleep(randomInt);
    		
			ChatMessageBuilder cm = new ChatMessageBuilder (CMD_ROOM_NAME_ACK, mRoomName);
			String message = cm.getMessage();
			byte[] send = message.getBytes();
			mChatService.write(send);
		} catch (Exception e) {
			e.printStackTrace();
		}
    }
    
    private void sendFindRoom(){
    	
    	try {
			ChatMessageBuilder mb = new ChatMessageBuilder(CMD_FIND_ROOM);
			String message = mb.getMessage();
			byte[] send = message.getBytes();
			mChatService.write(send);
		} catch (Exception e) {
			e.printStackTrace();
		}
    }

    
    /**
     * Sends a message.
     * @param message  A string of text to send.
     */
    private void sendMessage(String message) {
    	
    	if(mUserName == null){
    		Toast.makeText(this, "Please decide user name first", Toast.LENGTH_SHORT).show();
    		return;
    	}
    	
    	if(mRoomName == null){
    		Toast.makeText(this, "Please decide room first", Toast.LENGTH_SHORT).show();
    		return;
    	}    	
    	 
        // Check that there's actually something to send
        if (message.length() > 0 ) {
        	
        	message = mUserName +": "+ message ; 
        	
        	try {
        		
				ChatMessageBuilder mb = new ChatMessageBuilder(CMD_MESSAGE, mRoomName, message);
				message = mb.getMessage();
				
				// Get the message bytes and tell the BluetoothChatService to write
	            byte[] send = message.getBytes();
	            mChatService.write(send);

	            // Reset out string buffer to zero and clear the edit text field
	            mOutStringBuffer.setLength(0);
	            mOutEditText.setText(mOutStringBuffer);
	            
			} catch (Exception e) {
				e.printStackTrace();
			}
        }
    }

    
    private void findRoom(){
    	
    	try {
			Toast.makeText(this, "Try to find rooms .....", Toast.LENGTH_SHORT).show();
			existingRoomList = null;
			existingRoomList = new ArrayList<String>();
			FindRoomThread findT = new FindRoomThread();
			findT.start();
		} catch (Exception e) {
			
			e.printStackTrace();
		}
    }

    
    private void invokeFindRoomActivity(){
    	
    	Intent findRoomIntent = new Intent(this, FindRoomActivity.class);
	    findRoomIntent.putStringArrayListExtra( ROOM_LIST, existingRoomList);
	    startActivityForResult(findRoomIntent, INTENT_FIND_ROOM);
	    existingRoomList = null;
    }
    
    
    class FindRoomThread extends Thread{
    	
    	public void run(){
    		
    		try {
    			
    			sendFindRoom();
				Thread.sleep(FIND_ROOM_TIME_OUT); // wait 
				invokeFindRoomActivity();
			} catch (Exception e) {
				e.printStackTrace();
			}
    	}
    }
    
    
    // The action listener for the EditText widget, to listen for the return key
    private TextView.OnEditorActionListener mWriteListener =
        new TextView.OnEditorActionListener() {
        public boolean onEditorAction(TextView view, int actionId, KeyEvent event) {
            // If the action is a key-up event on the return key, send the message
            if (actionId == EditorInfo.IME_NULL && event.getAction() == KeyEvent.ACTION_UP) {
                String message = view.getText().toString();
                sendMessage(message);
            }
            if(D) Log.i(TAG, "END onEditorAction");
            return true;
        }
    };
    
    
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
    	
        if(D) Log.d(TAG, "onActivityResult " + resultCode);
        
        switch (requestCode) {
       
        case INTENT_GET_USERNAME:            
            // When user name input dialog returns
            if (resultCode == Activity.RESULT_OK ) {
            	
            	mUserName = data.getExtras()
                .getString(UserNameActivity.EXTRA_USER_NAME);
            	
            	Toast.makeText(this, "User name changed to "+mUserName, Toast.LENGTH_SHORT).show();
            	
            } else {
                
                Toast.makeText(this, R.string.username_error, Toast.LENGTH_SHORT).show();
            }
            break;
            
            // when create room dialog returns
            
        case INTENT_CREATE_ROOM:
        	if (resultCode == Activity.RESULT_OK ) {
            	
        		mRoomName = data.getExtras()
                .getString(CreateRoomActivity.EXTRA_ROOM_NAME);
            	
        		if(mRoomName.indexOf(DELIMITER) != -1){
        			mRoomName = mRoomName.replaceAll(DELIMITER, "!!");
        		}
        		
            	Toast.makeText(this, "Create a room : "+mRoomName, Toast.LENGTH_SHORT).show();
            	mTitle.setText(mRoomName);
            	
            } else {
                
                Toast.makeText(this, R.string.create_room_error, Toast.LENGTH_SHORT).show();
            } 
        	
        	break;
        
        case INTENT_FIND_ROOM:
        	if (resultCode == Activity.RESULT_OK ) {
	        	String mRoomToJoin = data.getExtras()
	            .getString(FindRoomActivity.EXTRA_JOIN_ROOM);
	        	
	        	if(mRoomToJoin != null && mRoomToJoin.length() > 0){
	        		
	        		if(mUserName != null)
	        			sendMessage(">>> "+mUserName +" leave !");
	        		
	        		mRoomName = mRoomToJoin;
	        		mTitle.setText(mRoomName);
	        		
	        		if(mUserName != null)
	        			sendMessage(">>> "+mUserName +" joined !");
	        	}
	        }
        	break;
        }// end switch
    }

    
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu, menu);
        return true;
    }


    public boolean onOptionsItemSelected(MenuItem item) {
    	
        switch (item.getItemId()) {
        
        case R.id.username:
            // Launch the UserNameActivity to see devices and do scan
            Intent userNameIntent = new Intent(this, UserNameActivity.class);
            startActivityForResult(userNameIntent, INTENT_GET_USERNAME); 
            return true;
            
        case R.id.createroom:
        	// Launch the CreateRoomActivity to see devices and do scan
            Intent createRoomIntent = new Intent(this, CreateRoomActivity.class);
            startActivityForResult(createRoomIntent, INTENT_CREATE_ROOM); 
        	return true;
        case R.id.findroom:
        	// Launch the CreateRoomActivity to see devices and do scan
        	
        	findRoom();
        	return true;
        }
        
        return false;
    }
    
    
    // The Handler that gets information back from the BluetoothChatService
    private final Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
        	
        	if(D) Log.d(TAG, "[handleMessage !!!!!!!!!!!! ]");
        	
            switch (msg.what) {
            
	            case MESSAGE_WRITE:
	          
					try {
						byte[] writeBuf = (byte[]) msg.obj;
		                // construct a string from the buffer
		                String writeMessage = new String(writeBuf);
		                ChatMessageChecker mc = new ChatMessageChecker (writeMessage);
		                
		                if(mc.getCmd().equals( CMD_MESSAGE ))
		                	mConversationArrayAdapter.add(mc.getMessageBody());
		                
					} catch (Exception e1) {
						e1.printStackTrace();
					} 
	                break;
	            case MESSAGE_READ:
	                String readBuf = (String) msg.obj;
	                
					try {
						
						ChatMessageChecker mc = new ChatMessageChecker(readBuf);
						
						if(mc.getCmd().equals(CMD_FIND_ROOM)){
							
							sendRoomNameAck();
							
						}else if(mc.getCmd().equals(CMD_ROOM_NAME_ACK)){
							
							if(existingRoomList != null){
								if(!existingRoomList.contains(mc.getRoomName()))
									existingRoomList.add(mc.getRoomName());
							}
							
						}else if(mc.getCmd().equals(CMD_MESSAGE)){
							if(mRoomName != null &&  mRoomName.equals(mc.getRoomName())){
								mConversationArrayAdapter.add(mc.getMessageBody());
							}else{
								if(D)Log.d(TAG,"Message from other room : "+mc.getRoomName()+"/"+mc.getMessageBody());
							}
						}
						
					} catch (Exception e) {
						e.printStackTrace();
					}
					
	                break;               
	            case MESSAGE_TOAST:
	                Toast.makeText(getApplicationContext(), msg.getData().getString(TOAST),
	                               Toast.LENGTH_SHORT).show();
	                break;
            }
        }
    };    
    

    class ChatMessageChecker{
    	String cmd 	= null;
    	String room = null;
    	String body = null;
    	
    	public ChatMessageChecker(String message) throws Exception{
    		
    		StringTokenizer st 	= new StringTokenizer(message,DELIMITER);
    		while(st.hasMoreElements()){
    			String token =  st.nextToken();
    			if(token.startsWith(CMD)){
    				cmd = token.substring(token.indexOf(CMD)+CMD.length());
    				continue;
    			}
    			
    			if(token.startsWith(ROOM)){
    				room = token.substring(token.indexOf(ROOM)+ROOM.length());
    				continue;
    			}
    			if(token.startsWith(BODY)){
    				body = message.substring(message.indexOf(BODY)+BODY.length());
    				continue;
    			}
    		}
    	}
    	
	    private String getRoomName(){
	    	return room;
	    } 
    
	    private String getCmd(){
	    	return cmd;
	    } 
    
	    private String getMessageBody(){
	    	return body;
	    }
    }
    
    
    class ChatMessageBuilder{
    	String message 	= null;
    	
    	public ChatMessageBuilder(String cmd) throws Exception{
    		this(cmd,null,null);
    	}
    	
    	public ChatMessageBuilder(String cmd, String roomName) throws Exception{
    		this(cmd,roomName,null);
    	}
    	
    	public ChatMessageBuilder(String cmd, String room, String body) throws Exception{    
    		if(cmd != null && cmd.equals(CMD_FIND_ROOM)){
    			message = 	CMD + CMD_FIND_ROOM;	
    		}else if (cmd != null && cmd.equals(CMD_ROOM_NAME_ACK) && room !=null){
    			message = 	CMD + CMD_ROOM_NAME_ACK + DELIMITER + ROOM + room;
    		}else if(cmd != null && room != null && body != null){
    			message = CMD + CMD_MESSAGE + DELIMITER + ROOM + room + DELIMITER + BODY +  body;
    		}
    	}
    	
	    private String getMessage(){
	    	return message;
	    }    
    }
}