/*
 * Copyright (C) 2009 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ncsu.course.android.broadcastchat4;

import java.util.ArrayList;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.res.Resources.NotFoundException;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

/**
 * This Activity appears as a dialog. It lists existing chat room
 * When a room is chosen by the user, the selected room name is sent back to the parent
 * Activity in the result Intent.
 */
public class FindRoomActivity extends Activity {
    // Debugging
    private static final String TAG = "FindRoomActivity";
    private static final boolean D = true;

    // Return Intent extra
    public static String EXTRA_JOIN_ROOM = "join_room";

    // Member fields
    private ArrayAdapter<String> mRoomListArrayAdapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Setup the window
        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
        setContentView(R.layout.find_room);  

        // Set result CANCELED in case the user backs out
        setResult(Activity.RESULT_CANCELED);

        
        // Initialize array adapters
        mRoomListArrayAdapter = new ArrayAdapter<String>(this, R.layout.room_name); 

        // Find and set up the ListView for paired devices
        ListView roomListView = (ListView) findViewById(R.id.find_room_listview);
        roomListView.setAdapter(mRoomListArrayAdapter);
        roomListView.setOnItemClickListener(mDeviceClickListener);

 
        
		ArrayList aList = null;
		
		Bundle extras = getIntent().getExtras();
		if(extras != null ){
			
			aList = extras.getStringArrayList(BroadcastChat.ROOM_LIST);	
		} 

		
		if (aList != null &&  0 < aList.size()) {
		    findViewById(R.id.find_room_listview).setVisibility(View.VISIBLE);
		    for(int i=0; aList != null && i< aList.size(); i++){
		    	mRoomListArrayAdapter.add(  aList.get(i).toString() );
		    }
		    
		} else {
		    String noRoom = getResources().getText(R.string.none_room_found).toString();
		    mRoomListArrayAdapter.add(noRoom);
		}
		
    } 

    
    // The on-click listener for all devices in the ListViews
    private OnItemClickListener mDeviceClickListener = new OnItemClickListener() {
        public void onItemClick(AdapterView<?> av, View v, int arg2, long arg3) {
             
            // Get the device MAC address, which is the last 17 chars in the View
            String roomName = ((TextView) v).getText().toString();
            
            // Create the result Intent and include the MAC address
            Intent intent = new Intent();
            intent.putExtra(EXTRA_JOIN_ROOM, roomName);

            // Set result and finish this Activity
            setResult(Activity.RESULT_OK, intent);
            finish();				// The ActivityResult is propagated back to whoever launched you via onActivityResult(). 
        }
    };

}
